# frozen_string_literal: true

module Sinamak
  begin
    CONNECTION = Faraday.new(url: Environment::URI) do |connection|
      connection.request :url_encoded
      connection.response :raise_error
      connection.adapter Faraday.default_adapter
    end
  rescue URI::InvalidURIError
    nil
  end

  class TranslationSource
    require_relative 'faraday/client_error'
    require_relative 'faraday/response'

    module ResponseCodes
      TRANSLATION = {
        OperationSuccess: 200,
        InvalidAPIKey: 401,
        BlockedAPIKey: 402,
        ExceedDailyLimit: 404,
        ExceedMaxTextSize: 413,
        CannotTranslate: 422,
        Unsupported: 501
      }.freeze
    end

    def initialize
      @connection = CONNECTION
    end

    def translate(text, from:, to:)
      request_translation(text, from, to).body_object.text
    end

    def detect_language(text)
      request_language(text).body_object.lang
    end

    def supported_languages
      supported_languages_response.body_object.dirs
    end

    private

    def noop_response
      @noop_response ||= NoopResponse.new
    end

    def request_translation(text, *lang)
      @connection.post('translate') do |request|
        request['Content-Type'] = 'application/x-www-form-urlencoded'
        request.body = { key: Environment::KEY, text: text.to_s, lang: lang.to_a.join('-') }
      end
    rescue Faraday::ClientError => e
      rescue_handle(e)
    end

    def request_language(text)
      @connection.post('detect') do |request|
        request['Content-Type'] = 'application/x-www-form-urlencoded'
        request.body = { key: Environment::KEY, text: text.to_s }
      end
    rescue Faraday::ClientError => e
      rescue_handle(e)
    end

    def request_supported_languages
      @connection.post('getLangs') do |request|
        request['Content-Type'] = 'application/x-www-form-urlencoded'
        request.body = { key: Environment::KEY, ui: 'en' }
      end
    rescue Faraday::ClientError => e
      rescue_handle(e)
    end

    def supported_languages_response
      @supported_languages_response ||= request_supported_languages
    end

    def rescue_handle(error)
      error.display_message
      noop_response
    end
  end
  private_constant :TranslationSource

  class NoopTranslationSource
    def detect_language(text)
    end

    def languages
    end
  end
  private_constant :NoopTranslationSource
end
